#!usr/bin/env python3

from hx711 import HX711
import sqlite3
import RPi.GPIO as GPIO
import sys
from os.path import dirname

AdmissibleError = 0.05

DbDirectory = dirname(__file__).split('/M')[0] + '/SetupInterface/database/Settings.db'
db = sqlite3.connect(DbDirectory)
Hive = int(sys.argv[1])
GPIODT = db.execute("SELECT GPIODT FROM Weight WHERE Hive == ?", (Hive, )).fetchone()
GPIOSCK = db.execute("SELECT GPIOSCK FROM Weight WHERE Hive == ?", (Hive, )).fetchone()
Channel = db.execute("SELECT Channel FROM Weight WHERE Hive == ?", (Hive, )).fetchone()
if GPIODT[0] and GPIOSCK[0] and GPIODT[0] != GPIOSCK[0]:
	OneWireDirectory = '/sys/bus/w1/devices/'
	ID = db.execute("SELECT TemperatureID FROM Weight WHERE Hive == ?",
								(Hive, )).fetchone()
	if ID[0]:
		try:
			Status = 'Reading'
			SensorFile = open(OneWireDirectory + ID[0] + '/w1_slave', 'r')
			SensorLines = SensorFile.readlines()
			SensorFile.close()
			Status = SensorLines[0].strip()[-3:]
		except:
			Temperature = 85
			print("Temperature Sensor for weight from hive number", Hive,
								"was not found")
		if Status == 'YES':
			MeasurePosition = SensorLines[1].find('t=')
			Temp = SensorLines[1][MeasurePosition + 2:]
			Temperature = float(Temp) / 1000
			if Temperature != 85:
				Offset = db.execute("SELECT TemperatureOffset FROM " \
								"Weight WHERE " \
								"Hive == ?",
								(Hive, )).fetchone()
				Ref = db.execute("SELECT TemperatureReference FROM " \
								"Weight WHERE " \
								"Hive == ?",
								(Hive, )).fetchone()
				Corr = db.execute("SELECT TemperatureCorrection FROM " \
								"Weight WHERE " \
								"Hive == ?",
								(Hive, )).fetchone()
				if Offset[0]:
					Temperature = Temperature - float(Offset[0])
				if Ref[0]:
					Temperature = Temperature - float(Ref[0])
				if Corr[0]:
					Correction = (Temperature * float(Corr[0]))
				else:
					Correction = 0
			else:
				print("Recieved error 85 for weight from hive number",
								Hive)
		elif Status == 'NO':
			Temperature = 85
			print("Temperature reading for weight from hive number", Hive,
								"is not a valid value")
	else:
		Temperature = 0
		Correction = 0
		print("A Temperature Sensor for weight from hive number", Hive,
								"was not selected")
	try:
		Ready = 0
		ResetSuccess = 0
		hx = HX711(dout_pin = GPIODT[0],
			   pd_sck_pin = GPIOSCK[0],
			   channel = Channel[0])
		hx.power_up()
		Ready = hx._ready()
		if Ready:
			ResetSuccess = hx.reset()
		ConnectionError = 0
	except:
		print("Connection error for HX711 from hive number", Hive)
		ConnectionError = 1
	if ResetSuccess and Temperature != 85:
		WeightMeasures = hx.get_raw_data(100)
		Readings = len(WeightMeasures)
		SortedMeasures = sorted(WeightMeasures)
		ReliableWeight = SortedMeasures[int(Readings/2)]
		ReliableMeasures = []
		AdmissibleError = ReliableWeight * AdmissibleError
		for Measure in range(Readings):
			Error = SortedMeasures[Measure] - ReliableWeight
			if abs(Error) < abs(AdmissibleError):
				ReliableMeasures.append(SortedMeasures[Measure])
		if len(ReliableMeasures) > 50:
			Weight = sum(ReliableMeasures)/len(ReliableMeasures)
			Offset = db.execute("SELECT Offset FROM Weight WHERE Hive == ?",
								(Hive, )).fetchone()
			Scale = db.execute("SELECT Scale FROM Weight WHERE Hive == ?",
								(Hive, )).fetchone()
			if Offset[0]:
				Weight = Weight - float(Offset[0])
			if Scale[0]:
				Weight = Weight / float(Scale[0])
			Weight = Weight + (Correction / 1000)
			Weight = round(Weight, 2)
			db.execute("UPDATE Weight SET Measurement = ? WHERE Hive = ?",
								(Weight, Hive))
			db.commit()
			print("Weight of hive number", str(Hive) + ":", Weight,"kg")
		elif len(ReliableMeasures) < 10:
			print("HX711 from hive number", Hive, "was not found")
		else:
			print("Not enough acceptable HX711 readings from hive number",
								Hive)
		hx.power_down()
	elif Temperature == 85:
		print("A Temperature Sensor for weight from hive number", Hive,
								"was selected, but",
								"ended with error")
	elif not Ready and not ConnectionError:
		print("HX711 from hive number", Hive, "was not found")
		hx.power_down()
	elif not ConnectionError:
		print("HX711 from hive number", Hive, "unable to reset")
		hx.power_down()
	GPIO.cleanup()
elif not GPIODT[0]:
	print("HX711 from hive number", Hive, "has not assigned DT GPIO")
elif not GPIOSCK[0]:
	print("HX711 from hive number", Hive, "has not assigned SCK GPIO")
else:
	print("Hx711 from hive number", Hive, "has DT and SCK assigned on the same GPIO")
db.close()
