#!/usr/bin/env python3

import RPi.GPIO as GPIO
import signal
import sqlite3
from os.path import dirname, exists
from time import sleep
from subprocess import run

LEDGPIO = 7
ButtonGPIO = 8
BlinkTimeMeasurementMode = 0.1
BlinkTimeSetupMode = 0.4
TimeON = BlinkTimeMeasurementMode
TimeOFF = TimeON
BounceTime = 12000

def CleanExit(sig, frame):
	GPIO.setup(LEDGPIO, GPIO.IN, pull_up_down=GPIO.PUD_UP)
	#run(["ip", "link", "set", "wlan0", "down"])
	#MeasurementMode = run(["systemctl", "is-active", "hostapd"])
	#if MeasurementMode.returncode != 0:
		#run(["wpa_cli", "-p", "/var/run/wpa_supplicant", "terminate"])
	#run(["dhclient", "-r", "wlan0"])

def ButtonPressedCallback(channel):
	global TimeON, TimeOFF
	TimeOFF = 0
	MeasurementMode = run(["systemctl", "is-active", "hostapd"])
	if MeasurementMode.returncode == 0:
		TimeON = BlinkTimeMeasurementMode
		StopSetupMode()
		StartMeasurementMode()
		TimeOFF = TimeON
		print("SweetHive is on Measurement Mode now")
	else:
		TimeON = BlinkTimeSetupMode
		print("Switching to Setup Mode...")
		StopMeasurementMode()
		StartSetupMode()
		TimeOFF = TimeON
		print("SweetHive is on Setup Mode now")

def StartMeasurementMode():
	#run(["wpa_supplicant", "-i", "wlan0", "-B",
				#"-c", "/etc/wpa_supplicant/wpa_supplicant.conf"])
	#run(["dhclient", "wlan0"])
	#run(["dhcpcd", "-n", "wlan0"])
	run(["systemctl", "start", "MeasurementMode"])

def StopMeasurementMode():
	run(["systemctl", "stop", "MeasurementMode"])
	run(["shutdown", "-c"])
	#run(["ip", "link", "set", "wlan0", "down"])
	#run(["wpa_cli", "-p", "/var/run/wpa_supplicant", "terminate"])
	#run(["systemctl", "stop", "wpa_supplicant"])
	#run(["dhclient", "-r", "wlan0"])

def StartSetupMode():
	run(["systemctl", "start", "SetupInterface"])
	run(["systemctl", "start", "hostapd"])
	run(["systemctl", "start", "dnsmasq"])
	run(["systemctl", "start", "nginx"])

def StopSetupMode():
	run(["ip", "link", "set", "wlan0", "down"])
	run(["systemctl", "stop", "nginx"])
	run(["systemctl", "stop", "SetupInterface"])
	run(["systemctl", "stop", "hostapd"])
	run(["systemctl", "stop", "dnsmasq"])

def ResetSensors():
	DbDirectory = dirname(__file__).split('/Services')[0] + \
						'/SetupInterface/database/Settings.db'
	if exists(DbDirectory):
		db = sqlite3.connect(DbDirectory)
		Hives = db.execute("SELECT Hive FROM Temperature").fetchall()
		for Hive in range(len(Hives)):
			db.execute("UPDATE Temperature SET Measurement = ? " \
						"WHERE Hive = ?", (None, Hive + 1))
			db.execute("UPDATE Temperature SET Sent = ? " \
						"WHERE Hive = ?", (0, Hive + 1))
			db.commit()
		Hives = db.execute("SELECT Hive FROM Humidity").fetchall()
		for Hive in range(len(Hives)):
			db.execute("UPDATE Humidity SET Measurement = ? " \
						"WHERE Hive = ?", (None, Hive + 1))
			db.execute("UPDATE Humidity SET Sent = ? " \
						"WHERE Hive = ?", (0, Hive + 1))
			db.commit()
		Hives = db.execute("SELECT Hive FROM Weight").fetchall()
		for Hive in range(len(Hives)):
			db.execute("UPDATE Weight SET Measurement = ? " \
						"WHERE Hive = ?", (None, Hive + 1))
			db.execute("UPDATE Weight SET Sent = ? " \
						"WHERE Hive = ?", (0, Hive + 1))
			db.commit()
		db.close()
	else:
		print("Settings database was not found")

if __name__ == '__main__':
	for Signal in [signal.SIGTERM, signal.SIGINT]:
		signal.signal(Signal, CleanExit)
	GPIO.setmode(GPIO.BCM)
	GPIO.setup(ButtonGPIO, GPIO.IN, pull_up_down=GPIO.PUD_UP)
	GPIO.setup(LEDGPIO, GPIO.OUT, initial=GPIO.HIGH)
	StopSetupMode()
	ResetSensors()
	StartMeasurementMode()
	GPIO.add_event_detect(ButtonGPIO, GPIO.FALLING,
			callback=ButtonPressedCallback, bouncetime=BounceTime)
	print("SweetHive has been fully loaded and ready to work")
	while True:
		GPIO.output(LEDGPIO, GPIO.LOW)
		sleep(TimeOFF)
		GPIO.output(LEDGPIO, GPIO.HIGH)
		sleep(TimeON)
