#!/usr/bin/env python3

from flask import Flask, redirect, render_template, request, json, send_file
from flask_sqlalchemy import SQLAlchemy
from os.path import dirname, join
from os import listdir
from subprocess import run

UPLOAD_FOLDER = dirname(__file__) + '/database'

app = Flask(__name__)
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///database/Settings.db'
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER
db = SQLAlchemy(app)
MaxHives = 6
MaxThingSpeakChannels = 20
ScrollingTo = ''

class TS(db.Model):
    ChannelNumber = db.Column(db.INTEGER, primary_key = True)
    ChannelName = db.Column(db.TEXT)
    ChannelID = db.Column(db.TEXT)
    ChannelWriteAPIKey = db.Column(db.TEXT)

class Temperature(db.Model):
    Hive = db.Column(db.INTEGER, primary_key = True)
    Enabled = db.Column(db.INTEGER)
    ID = db.Column(db.TEXT)
    ThingSpeakChannelNumber = db.Column(db.INTEGER)
    ThingSpeakChannelField = db.Column(db.INTEGER)
    Offset = db.Column(db.REAL)
    Measurement = db.Column(db.REAL)
    Sent = db.Column(db.INTEGER)

class Humidity(db.Model):
    Hive = db.Column(db.INTEGER, primary_key = True)
    Enabled = db.Column(db.INTEGER)
    GPIO = db.Column(db.INTEGER)
    ThingSpeakChannelNumber = db.Column(db.INTEGER)
    ThingSpeakChannelField = db.Column(db.INTEGER)
    Offset = db.Column(db.REAL)
    Measurement = db.Column(db.REAL)
    Sent = db.Column(db.INTEGER)

class Weight(db.Model):
    Hive = db.Column(db.INTEGER, primary_key = True)
    Enabled = db.Column(db.INTEGER)
    GPIODT = db.Column(db.INTEGER)
    GPIOSCK = db.Column(db.INTEGER)
    Channel = db.Column(db.TEXT)
    ThingSpeakChannelNumber = db.Column(db.INTEGER)
    ThingSpeakChannelField = db.Column(db.INTEGER)
    Offset = db.Column(db.REAL)
    Scale = db.Column(db.REAL)
    TemperatureID = db.Column(db.TEXT)
    TemperatureOffset = db.Column(db.REAL)
    TemperatureReference = db.Column(db.REAL)
    TemperatureCorrection = db.Column(db.REAL)
    Measurement = db.Column(db.REAL)
    Sent = db.Column(db.INTEGER)

class W1(db.Model):
    PrimaryKey = db.Column(db.INTEGER, primary_key = True)
    SensorID = db.Column(db.TEXT)

class AvailableGPIO(db.Model):
    GPIO = db.Column(db.INTEGER, primary_key = True)
    Available = db.Column(db.INTEGER)

db.create_all()

def FillExpectedHives():
    for Hives in range(MaxHives):
        if not Temperature.query.get(Hives + 1):
            Sensor = Temperature(Hive = Hives + 1, Enabled = 0, Sent = 0)
            db.session.add(Sensor)
        if not Humidity.query.get(Hives + 1):
            Sensor = Humidity(Hive = Hives + 1, Enabled = 0, Sent = 0)
            db.session.add(Sensor)
        if not Weight.query.get(Hives + 1):
            Sensor = Weight(Hive = Hives + 1, Enabled = 0, Sent = 0)
            db.session.add(Sensor)
    db.session.commit()

def CheckAvailableGPIO():
    for GPIO in range(28):
        if GPIO not in [0, 1, 2, 3, 4, 7, 8, 14, 17, 27]:
            if not AvailableGPIO.query.get(GPIO):
                IO = AvailableGPIO(GPIO = GPIO)
                db.session.add(IO)
            IO = AvailableGPIO.query.get(GPIO)
            if Humidity.query.filter_by(Enabled = 1, GPIO = GPIO).first():
                IO.Available = 0
            elif Weight.query.filter_by(Enabled = 1, GPIODT = GPIO).first():
                IO.Available = 0
            elif Weight.query.filter_by(Enabled = 1, GPIOSCK = GPIO).first():
                IO.Available = 0
            else:
                IO.Available = 1
    db.session.commit()

LanguagesFiles = listdir(dirname(__file__) + '/language')
Languages = [Files.split('.')[0] for Files in LanguagesFiles]

try:
    with open(dirname(__file__) + '/database/SelectedLanguage.json', 'r', encoding = 'utf8') as language_file:
        language = json.load(language_file)
        language_file.close()
except:
    with open(dirname(__file__) + '/language/DefaultLanguage.json', 'r', encoding = 'utf8') as language_file:
        language = json.load(language_file)
        language_file.close()

Accordion = '{ "ThingSpeak": false, "Language": false, "Schedule": false, "SettingsBackup": false'
for Heading in range(MaxHives):
    Accordion = Accordion + ', "Hive' + str(Heading + 1) + '": false'
Accordion = Accordion + ' }'
Accordion = json.loads(Accordion)

with open(dirname(__file__) + '/database/AccordionState.json', 'w') as accordion_file:
    json.dump(Accordion, accordion_file)
    accordion_file.close()

try:
    Schedule = open(dirname(__file__).split('/Setup')[0] + '/wittypi/schedule.wpi', 'r').read()
    open(dirname(__file__) + '/database/schedule.wpi', 'w').write(Schedule)
except:
    try:
        ScheduleFile = open(dirname(__file__) + '/database/schedule.wpi', 'r').read().split()
        for Word in range(len(ScheduleFile)):
            if Word == 100:
                break
            elif Word == 0:
                Schedule = ScheduleFile[Word]
            elif ScheduleFile[Word] in ['END', 'ON', 'OFF', 'BEGIN']:
                Schedule = Schedule + '\n' + ScheduleFile[Word]
            else:
                Schedule = Schedule + '\t' + ScheduleFile[Word]
        open(dirname(__file__).split('/Setup')[0] + '/wittypi/schedule.wpi', 'w').write(Schedule)
    except:
        Schedule = ''
        open(dirname(__file__) + '/database/schedule.wpi', 'w').write(Schedule)

FillExpectedHives()
CheckAvailableGPIO()
ThingSpeakChannels = TS.query.all()
TemperatureSensors = Temperature.query.all()
HumiditySensors = Humidity.query.all()
WeightSensors = Weight.query.all()
GPIO = AvailableGPIO.query.all()
DS18B20SensorID = W1.query.all()

@app.route('/')
def index():
    return render_template('index.html',
                           MaxHives = MaxHives,
                           MaxThingSpeakChannels = MaxThingSpeakChannels,
                           ScrollingTo = ScrollingTo,
                           Languages = sorted(Languages),
                           language = language,
                           Accordion = Accordion,
                           ThingSpeakChannels = ThingSpeakChannels,
                           TemperatureSensors = TemperatureSensors,
                           HumiditySensors = HumiditySensors,
                           WeightSensors = WeightSensors,
                           GPIO = GPIO,
                           DS18B20SensorID = DS18B20SensorID,
                           Schedule = Schedule)

@app.route('/AddThingSpeakChannel', methods = ['POST'])
def AddThingSpeakChannel():
    global ScrollingTo, ThingSpeakChannels
    for LowestAvailablePrimaryKey in range(MaxThingSpeakChannels):
        if not TS.query.get(LowestAvailablePrimaryKey + 1):
            LowestAvailablePrimaryKey += 1
            break
    ScrollingTo = 'ThingSpeak' + str(LowestAvailablePrimaryKey)  
    ThingSpeakChannel = TS(ChannelNumber = LowestAvailablePrimaryKey,
                           ChannelName = request.form['FormThingSpeakChannelName'],
                           ChannelID = request.form['FormThingSpeakChannelID'],
                           ChannelWriteAPIKey = request.form['FormThingSpeakChannelWriteAPIKey'])                          
    db.session.add(ThingSpeakChannel)
    db.session.commit()
    ThingSpeakChannels = TS.query.all()
    return redirect('/')

@app.route('/EditThingSpeakChannel/<ThingSpeakChannelNumber>', methods = ['POST'])
def EditThingSpeakChannel(ThingSpeakChannelNumber):
    global ScrollingTo, ThingSpeakChannels, TemperatureSensors, HumiditySensors, WeightSensors
    ScrollingTo = 'ThingSpeak' + str(ThingSpeakChannelNumber)
    ThingSpeakChannel = TS.query.get(ThingSpeakChannelNumber)
    ThingSpeakChannel.ChannelName = request.form['FormThingSpeakChannelName']
    ThingSpeakChannel.ChannelID = request.form['FormThingSpeakChannelID']
    ThingSpeakChannel.ChannelWriteAPIKey = request.form['FormThingSpeakChannelWriteAPIKey']
    for Hives in range(MaxHives):
        TemperatureSensor = Temperature.query.get(Hives + 1)
        if TemperatureSensor.ThingSpeakChannelNumber == ThingSpeakChannel.ChannelNumber:
            TemperatureSensor.Sent = 0
        HumiditySensor = Humidity.query.get(Hives + 1)
        if HumiditySensor.ThingSpeakChannelNumber == ThingSpeakChannel.ChannelNumber:
            HumiditySensor.Sent = 0
        WeightSensor = Weight.query.get(Hives + 1)
        if WeightSensor.ThingSpeakChannelNumber == ThingSpeakChannel.ChannelNumber:
            WeightSensor.Sent = 0
    db.session.commit()
    ThingSpeakChannels = TS.query.all()
    TemperatureSensors = Temperature.query.all()
    HumiditySensors = Humidity.query.all()
    WeightSensors = Weight.query.all()
    return redirect('/')

@app.route('/DeleteThingSpeakChannel/<ThingSpeakChannelNumber>')
def DeleteThingSpeakChannel(ThingSpeakChannelNumber):
    global ScrollingTo, ThingSpeakChannels, TemperatureSensors, HumiditySensors, WeightSensors, GPIO
    ScrollingTo = 'HeadingThingSpeak'
    ThingSpeakChannel = TS.query.get(ThingSpeakChannelNumber)
    for Hives in range(MaxHives):
        TemperatureSensor = Temperature.query.get(Hives + 1)
        if TemperatureSensor.ThingSpeakChannelNumber == ThingSpeakChannel.ChannelNumber:
            TemperatureSensor.Enabled = 0
            TemperatureSensor.ThingSpeakChannelNumber = None
            TemperatureSensor.Sent = 0
        HumiditySensor = Humidity.query.get(Hives + 1)
        if HumiditySensor.ThingSpeakChannelNumber == ThingSpeakChannel.ChannelNumber:
            HumiditySensor.Enabled = 0
            HumiditySensor.ThingSpeakChannelNumber = None
            HumiditySensor.Sent = 0
        WeightSensor = Weight.query.get(Hives + 1)
        if WeightSensor.ThingSpeakChannelNumber == ThingSpeakChannel.ChannelNumber:
            WeightSensor.Enabled = 0
            WeightSensor.ThingSpeakChannelNumber = None
            WeightSensor.Sent = 0
    db.session.delete(ThingSpeakChannel)
    db.session.commit()
    CheckAvailableGPIO()
    ThingSpeakChannels = TS.query.all()
    TemperatureSensors = Temperature.query.all()
    HumiditySensors = Humidity.query.all()
    WeightSensors = Weight.query.all()
    GPIO = AvailableGPIO.query.all()
    return redirect('/')

@app.route('/EditTemperatureSensor/<Hive>', methods = ['POST'])
def EditTemperatureSensor(Hive):
    global ScrollingTo, TemperatureSensors
    ScrollingTo = 'Hive' + str(Hive) + 'Temperature'
    TemperatureSensor = Temperature.query.get(Hive)
    TemperatureSensor.Enabled = request.form['FormTemperatureSensorEnabled']
    TemperatureSensor.ID = request.form['FormTemperatureSensorID']
    TemperatureSensor.ThingSpeakChannelNumber = request.form['FormTemperatureSensorThingSpeakChannel']
    TemperatureSensor.ThingSpeakChannelField = request.form['FormTemperatureSensorThingSpeakChannelField']
    if request.form['FormTemperatureSensorOffset']:
        TemperatureSensor.Offset = request.form['FormTemperatureSensorOffset']
    else:
        TemperatureSensor.Offset = 0.0
    TemperatureSensor.Measurement = None
    TemperatureSensor.Sent = 0
    db.session.commit()
    TemperatureSensors = Temperature.query.all()
    return redirect('/')

@app.route('/EditHumiditySensor/<Hive>', methods = ['POST'])
def EditHumiditySensor(Hive):
    global ScrollingTo, HumiditySensors, WeightSensors, GPIO
    ScrollingTo = 'Hive' + str(Hive) + 'Humidity'
    HumiditySensor = Humidity.query.get(Hive)
    HumiditySensor.Enabled = request.form['FormHumiditySensorEnabled']
    HumiditySensor.GPIO = request.form['FormHumiditySensorGPIO']
    HumiditySensor.ThingSpeakChannelNumber = request.form['FormHumiditySensorThingSpeakChannel']
    HumiditySensor.ThingSpeakChannelField = request.form['FormHumiditySensorThingSpeakChannelField']
    if request.form['FormHumiditySensorOffset']:
        HumiditySensor.Offset = request.form['FormHumiditySensorOffset']
    else:
        HumiditySensor.Offset = 0.0
    HumiditySensor.Measurement = None
    HumiditySensor.Sent = 0
    db.session.commit()
    if HumiditySensor.Enabled:
        GPIO = HumiditySensor.GPIO
        Hive = HumiditySensor.Hive
        for Hives in range(MaxHives):
            if Hives + 1 != Hive:
                HumiditySensor = Humidity.query.get(Hives + 1)
                if HumiditySensor.GPIO == GPIO:
                    HumiditySensor.GPIO = None
            WeightSensor = Weight.query.get(Hives + 1)
            if WeightSensor.GPIODT == GPIO:
                WeightSensor.GPIODT = None
            if WeightSensor.GPIOSCK == GPIO:
                WeightSensor.GPIOSCK = None
        db.session.commit()
    CheckAvailableGPIO()
    HumiditySensors = Humidity.query.all()
    WeightSensors = Weight.query.all()
    GPIO = AvailableGPIO.query.all()
    return redirect('/')

@app.route('/EditWeightSensor/<Hive>', methods = ['POST'])
def EditWeightSensor(Hive):
    global ScrollingTo, HumiditySensors, WeightSensors, GPIO
    ScrollingTo = 'Hive' + str(Hive) + 'Weight'
    WeightSensor = Weight.query.get(Hive)
    WeightSensor.Enabled = request.form['FormWeightSensorEnabled']
    WeightSensor.GPIODT = request.form['FormWeightSensorGPIODT']
    WeightSensor.GPIOSCK = request.form['FormWeightSensorGPIOSCK']
    WeightSensor.Channel = request.form['FormWeightSensorChannel']
    WeightSensor.ThingSpeakChannelNumber = request.form['FormWeightSensorThingSpeakChannel']
    WeightSensor.ThingSpeakChannelField = request.form['FormWeightSensorThingSpeakChannelField']
    if request.form['FormWeightSensorOffset']:
        WeightSensor.Offset = request.form['FormWeightSensorOffset']
    else:
        WeightSensor.Offset = 0.0
    if request.form['FormWeightSensorScale']:
        WeightSensor.Scale = request.form['FormWeightSensorScale']
    else:
        WeightSensor.Scale = 1.0
    WeightSensor.TemperatureID = request.form['FormWeightTemperatureID']
    if request.form['FormWeightTemperatureOffset']:
        WeightSensor.TemperatureOffset = request.form['FormWeightTemperatureOffset']
    else:
        WeightSensor.TemperatureOffset = 0.0
    if request.form['FormWeightTemperatureReference']:
        WeightSensor.TemperatureReference = request.form['FormWeightTemperatureReference']
    else:
        WeightSensor.TemperatureReference = 0.0
    if request.form['FormWeightTemperatureCorrection']:
        WeightSensor.TemperatureCorrection = request.form['FormWeightTemperatureCorrection']
    else:
        WeightSensor.TemperatureCorrection = 0.0
    WeightSensor.Measurement = None
    WeightSensor.Sent = 0
    db.session.commit()
    if WeightSensor.Enabled:
        GPIODT = WeightSensor.GPIODT
        GPIOSCK = WeightSensor.GPIOSCK
        Hive = WeightSensor.Hive
        for Hives in range(MaxHives):
            HumiditySensor = Humidity.query.get(Hives + 1)
            if HumiditySensor.GPIO == GPIODT:
                HumiditySensor.GPIO = None
            elif HumiditySensor.GPIO == GPIOSCK:
                HumiditySensor.GPIO = None
            if Hives + 1 != Hive:
                WeightSensor = Weight.query.get(Hives + 1)
                if WeightSensor.GPIODT == GPIODT:
                    WeightSensor.GPIODT = None
                elif WeightSensor.GPIODT == GPIOSCK:
                    WeightSensor.GPIODT = None
                if WeightSensor.GPIOSCK == GPIODT:
                    WeightSensor.GPIOSCK = None
                elif WeightSensor.GPIOSCK == GPIOSCK:
                    WeightSensor.GPIOSCK = None
        db.session.commit()
    CheckAvailableGPIO()
    HumiditySensors = Humidity.query.all()
    WeightSensors = Weight.query.all()
    GPIO = AvailableGPIO.query.all()
    return redirect('/')

@app.route('/UpdateAccordionPanels/<Panel>')
def UpdateAccordionPanels(Panel):
    global ScrollingTo, Accordion
    ScrollingTo = 'Heading' + str(Panel)
    with open(dirname(__file__) + '/database/AccordionState.json', 'r+') as accordion_file:
        Accordion = json.load(accordion_file)
        Accordion[Panel] = not Accordion[Panel]
        accordion_file.seek(0)
        json.dump(Accordion, accordion_file)
        accordion_file.truncate()
        accordion_file.close()
    return redirect('/')

@app.route('/ChangeLanguage/<Language>')
def ChangeLanguage(Language):
    global ScrollingTo, language, Accordion
    ScrollingTo = ''
    with open(dirname(__file__) + '/language/' + Language + '.json', 'r', encoding = 'utf8') as language_file:
        language = json.load(language_file)
        language_file.close()
    with open(dirname(__file__) + '/database/SelectedLanguage.json', 'w', encoding = 'utf8') as language_file:
        json.dump(language, language_file)
        language_file.close()
    with open(dirname(__file__) + '/database/AccordionState.json', 'r+') as accordion_file:
        Accordion = json.load(accordion_file)
        Accordion['Language'] = not Accordion['Language']
        accordion_file.seek(0)
        json.dump(Accordion, accordion_file)
        accordion_file.truncate()
        accordion_file.close()
    return redirect('/')

@app.route('/Execute/<Variable>/<Hive>/<Script>')
def Execute(Variable, Hive, Script):
    global ScrollingTo, DS18B20SensorID, TemperatureSensors, HumiditySensors, WeightSensors
    ScrollingTo = 'Hive' + str(Hive) + Variable
    ScriptDirectory = dirname(__file__).split('/Setup')[0] + '/MeasurementScripts/' + Script
    run(["sudo", "python", ScriptDirectory, Hive])  
    if Variable == 'Temperature':
        if Script == 'SearchDS18B20ID.py':
            DS18B20SensorID = W1.query.all()
        else:
            TemperatureSensors = Temperature.query.all()
    if Variable == 'Humidity':
        HumiditySensors = Humidity.query.all()
    if Variable == 'Weight': 
        WeightSensors = Weight.query.all()
    return redirect('/')

@app.route('/ChangeSchedule', methods = ['POST'])
def ChangeSchedule():
    global ScrollingTo, Schedule
    ScrollingTo = 'HeadingSchedule'
    ScheduleFile = request.form['FormScheduleRegister'].split()
    for Word in range(len(ScheduleFile)):
        if Word == 100:
            break
        elif Word == 0:
            Schedule = ScheduleFile[Word]
        elif ScheduleFile[Word] in ['END', 'ON', 'OFF', 'BEGIN']:
            Schedule = Schedule + '\n' + ScheduleFile[Word]
        else:
            Schedule = Schedule + '\t' + ScheduleFile[Word]
    open(dirname(__file__) + '/database/schedule.wpi', 'w').write(Schedule)
    open(dirname(__file__).split('/Setup')[0] + '/wittypi/schedule.wpi', 'w').write(Schedule)
    return redirect('/')

@app.route('/Download/<File>')
def Download(File):
    Path = dirname(__file__) + '/database/' + File
    return send_file(Path, as_attachment=True)

@app.route('/UploadSchedule', methods = ['POST'])
def UploadSchedule():
    global ScrollingTo, Schedule
    ScrollingTo = 'HeadingSchedule'
    if 'FormScheduleUpload' not in request.files:
        return redirect('/')
    ScheduleFile = request.files['FormScheduleUpload']
    if ScheduleFile:
        ScheduleFile.save(join(app.config['UPLOAD_FOLDER'], 'schedule.wpi'))
        ScheduleFile = open(dirname(__file__) + '/database/schedule.wpi', 'r').read().split()
        for Word in range(len(ScheduleFile)):
            if Word == 100:
                break
            elif Word == 0:
                Schedule = ScheduleFile[Word]
            elif ScheduleFile[Word] in ['END', 'ON', 'OFF', 'BEGIN']:
                Schedule = Schedule + '\n' + ScheduleFile[Word]
            else:
                Schedule = Schedule + '\t' + ScheduleFile[Word]
    return redirect('/')

@app.route('/UploadSettingsBackup', methods = ['POST'])
def UploadSettingsBackup():
    global ScrollingTo, ThingSpeakChannels, TemperatureSensors, HumiditySensors, WeightSensors, GPIO, DS18B20SensorID, Accordion
    ScrollingTo = 'HeadingSettingsBackup'
    if 'FormSettingsBackupUpload' not in request.files:
        return redirect('/')
    SettingsBackup = request.files['FormSettingsBackupUpload']
    if SettingsBackup:
        SettingsBackup.save(join(app.config['UPLOAD_FOLDER'], 'Settings.db'))
        db.create_all()
        FillExpectedHives()
        CheckAvailableGPIO()
        ThingSpeakChannels = TS.query.all()
        TemperatureSensors = Temperature.query.all()
        HumiditySensors = Humidity.query.all()
        WeightSensors = Weight.query.all()
        GPIO = AvailableGPIO.query.all()
        DS18B20SensorID = W1.query.all()
        with open(dirname(__file__) + '/database/AccordionState.json', 'r+') as accordion_file:
            Accordion = json.load(accordion_file)
            Accordion['SettingsBackup'] = not Accordion['SettingsBackup']
            accordion_file.seek(0)
            json.dump(Accordion, accordion_file)
            accordion_file.truncate()
            accordion_file.close()
        ScrollingTo = ''
    return redirect('/')

if __name__ == '__main__':
    app.run(debug = True)